% loadEquations converts an ODE model to Boolean model using the systematic transition scheme 
% presented in:

% Sttzel C., Rblitz S., and Siebert H. (2015), Complementing ODE-Based System Analysis 
% Using Boolean Networks Derived from an Euler-like Transformation, 
% PLoS ONE 10(10): e0140954. doi:10.1371/journal.pone.0140954

% model=loadEquations(ode) takes as input a MATLAB ODE program file or a
% text file, where each monotone function in the rhs of each equation is
% parenthesized

% the text file must contain one ordinary differential equation per line;
%   the parameter values are substituted in the right hand side functions;
%   each equation in the text file has the form: dvarname = rhs;

% the MATLAB ODE file must contain the key words: parameters, end of parameters, 
%   equations and end of equations 


function model = loadEquations(ode)
if ischar(ode)
    if isODEscript(ode)
        model = loadODEscript(ode);
    else
        % if the input is a text file, read data line by line and store each in cell array
        eq_cell = readTextFile(ode);
        var_cell = cell(size(eq_cell));
        rhs_cell = cell(size(eq_cell));
        % extract variables and corresponding equations into two cell arrays
        for i=1:numel(eq_cell) 
            eq_sign = strfind(eq_cell{i}, '=');
   
            var_cell{i}=strtrim(eq_cell{i}(2:eq_sign-1)); %the first character in variables is a 'd'
            rhs_cell{i}=strtrim(eq_cell{i}(eq_sign+1:end));
        end

        model.species = var_cell;  
        model.rhs = rhs_cell;
    end

end


 
% pattern for expressions '(..)/(..)' or '((..)/(..))' in the rhs
fract = '\([a-z0-9+\-*.\s^]+\)/\([a-z0-9+\-*.\s^]+\)';

% pattern for remaining expressions '(..)' in the rhs
within_paranth = '\([a-z0-9+\-*.\s\^]+\)';

% fr_expr is a cell array containing all fraction expressions '(..)/(..)'
fr_expr=cellfun(@(x) regexp(x,fract,'match'), model.rhs,'UniformOutput' ,false);


%fr_expr=flatten(fr_expr);
fr_expr=[fr_expr{:}];   %flattens nested cell array


% temp is the rhs cell array where the '(..)/(..)' are replaced by '#'
temp=cellfun(@(x) rep_fraction(x,fract), model.rhs,'UniformOutput' ,false);
%celldisp(temp)

% par_expr is a cell array containing all the expressions within parantheses: '(..)'
par_expr=cellfun(@(x) regexp(x,within_paranth,'match'), temp,'UniformOutput' ,false);
%par_expr=flatten(par_expr);
par_expr=[par_expr{:}];



function var = get_var(str)
% returns the continuous variable, contained in a function
% only one variable per function allowed, otherwise print error message
var={};

for j=1:numel(model.species)
  if(~isempty(strfind(str,model.species{j})))
      var=[var model.species{j}];
  end
end
 if (numel(var)>1) % more than one continuous variable matched in rhs function
     error('Error. The math expression %s must contain only one varible.' , str);
 elseif (numel(var)==0) % no continuous variable matched in rhs expression
         error('Error. The math expression %s does not contain any input varible.' , str)
 else
      var=char(var); %var is a cell array of one string, convert it to type string
 end
end


 
function str = func_rep(eq,mon_fnc)
% replaces a monotone function: mon_fnc by its descrete counterpart
% depending on whether the the function is monotonically decreasing 
% or monotonically increasing

    s = monotonic(mon_fnc,get_var(mon_fnc)); %a string with values 'incr' or 'decr'
    boolVar = get_boolVar(get_var(mon_fnc)); %the discrete counterpart of the variable
    
    if strcmp(s,'incr') == 1
        str = strrep(eq,mon_fnc,boolVar); %monotonically increasing function mapping
                                       
    else
        str = strrep(eq,mon_fnc,['(1-' boolVar ')']); %monotonically decreasing function
                                             
    end
end


function str = mult_rep(eq,cell_func)
% applies the function func_rep on expression eq, the first input argument, 
% replacing multiple monotone functions, given as a second input argument (cell_func)
     str=eq;
     for t=1:numel(cell_func)
        str=func_rep(str,cell_func{t});
     end
end


function var_new = var_rep(var_old)
% replaces continuous variables in equation with the corresponding discrete variables
% input is the variable name
     var_new=var_old;
     for k=1:numel(model.species)
         var_new=strrep(var_new,model.species{k},get_boolVar(model.species{k}));
     end
    
end

%cell array, where each monotonically function (..)/(..) is replaced by the
%discrete counterpart
b_rhs=cellfun(@(x) mult_rep(x,fr_expr), model.rhs,'UniformOutput' ,false);

%every monotonically function within '(..)' is replaced according to its monotonicity
bool_rhs=cellfun(@(x) mult_rep(x,par_expr), b_rhs,'UniformOutput' ,false);

%replace remaining continuous varibales by the corresponding discrete
%variables
bool_rhs=cellfun(@(x) var_rep(x), bool_rhs,'UniformOutput' ,false);

%each real valued variable is represented by a corresponding discrete variable
bool_sp=cellfun(@(x) var_rep(x), model.species,'UniformOutput' ,false);

model.bool_species=bool_sp;  
                            
model.bool_rhs=bool_rhs;

% celldisp(model.rhs)
celldisp((model.bool_rhs))


end


function m=rep_fraction(str,pattern)
% matches pattern in input str and replaces it with #

l = regexp(str,pattern,'match');
m=str;
for i=1:numel(l)
    m=strrep(m,l{i},'#');
end

end



function val = monotonic(fnc,var)
% determines if the expression in the input function: fnc is a monotonically increasing
% or monotonically decreasing function of the variable var (the second input argument)
% output argument of type string: 'incr' or 'decr'
    try
        m = subs(fnc,var,'0.5');
    catch
        m = subs(fnc,var,'0.4');
    end
    try
        n = subs(fnc,var, '1');
    catch
        n = subs(fnc,var, '0.9');
    end
    if (m <= n)
        val = 'incr';
    else
        val = 'decr';
    end
end


 function var = get_boolVar(var_old)
 % replaces the continuous variable with the corresponding discrete
 % variable
    var= regexprep(var_old,'[a-zA-Z]','x');
 end
 


function cell_arr = readTextFile(txt_file)
% function to read in a text file line per line and write the lines to a cell array
fid=fopen(txt_file,'r');
cell_arr={};
while ~feof(fid) 
   cell_arr{end + 1}=fgetl(fid);
end
fclose(fid);
end
 


 

