% ODETOBNET generates boolean expressions from differential equations
%
%   input is a text file or a MATLAB ODE program file, containing an ODE system

%   first calls the helper function loadEquations, which converts the ODEs 
%   to Boolean equations using the systematic transition scheme

%   the values of the functions in the rhs are evaluated, depending on the values
%   of the Boolean variables, stored in a truth table 

%   the he resulting updating rules are written to file: model.bnet,
%   which has the file format required by the BoolNet R package:
%   BoolNet is a software tool for for assembling, analyzing and visualizing 
%   synchronous and asynchronous Boolean networks as well as probabilistic
%   Boolean networks

%   Reference:
%   BoolNet--an R package for generation, reconstruction and analysis of
%   Boolean networks.
%   Bioinformatics. 2010 May 15;26(10):1378-80. doi:
%   10.1093/bioinformatics/btq124. Epub 2010 Apr 7.

function odeToBNet(eq)

%load equations using the function loadEquations
model = loadEquations(eq); 


% open file to write the equations in BoolNet file format 
fileID=fopen('model.bnet','w');

fprintf(fileID,'%6s, %s\n','targets','factors');

for m=1:numel(model.bool_rhs)
    cur_var=model.bool_species{m}; % current discrete variable 

    rhs=model.bool_rhs{m}; %current function
   
    varInFnc = get_var(rhs); %discrete variables, occuring in the current function
    
    ind = find(strcmp(cur_var, varInFnc)); %the index of current var in the array
    
    %if current var not occuring in array (uncoupled variable)
    if isempty(ind)
        ind = numel(varInFnc) + 1;
        varInFnc{end + 1}=cur_var; %insert current variable at the end of the array
      
    end
    
    
    %get a matrix of size 2^numel(varInFnc) x numel(varInFnc)
    tab = get_tTable(numel(varInFnc)); 
    
    %create a truth table of desired size (see above)
    T=array2table(tab);
    T.Properties.VariableNames = varInFnc; %set the variable names as colomn names of the table
 
    
    tab_row = num2cell(tab,2); %each cell contains a row of the truth table tab
   
    % substitutes the values of the variables in the current function
    % the h function in the corresponding paper
    fnc_h=cellfun(@(x) subsTab(x,rhs), tab_row,'UniformOutput' ,false);
    
 
    % a vector containing the values of the sign function 
    sgn = sign(double([fnc_h{:}])); 
  
    tab_col = (num2cell(tab,1)); 
   
    cur_col = tab_col{ind}; %the values of the cur variables in the truth table
    
    update = g(sgn,cur_col);
    
    non_zero = find(update); %the indices of non zero entries in the update vector
    
    % according to the values in the update vector, a logical expression is
    % determined and written in the output file
    str_eq = {};
    
    for i=1:numel(non_zero)
        row=T(non_zero(i),:); %returns row of T with value 1 in the update vector
        cell_arr = get_LogicVar(row,varInFnc); %get the variables
        %disp(get_LogicExpr(row,var));
        str = strjoin(cell_arr,' & '); % joins the variables with logical AND
        %disp(str)
        str_eq=[str_eq str];
    end
    
    % write the logical equation in the text file, joined by logical OR
    fprintf(fileID,'%s, %s\n',cur_var,strjoin(str_eq,' | '));
    
    % optional:
    % displays the truth tables 
    col = table(update');
    cur_name = cellstr(strcat(varInFnc{ind},'_next')); %renames the cur var
    col.Properties.VariableNames = cur_name;
    sgn_col = table(sgn');
    sgn_col.Properties.VariableNames = cellstr('sgn');
    T = [T sgn_col col]; 
    disp(T); %prints current table T
     
 end
fclose(fileID);



function var = get_var(str)
% returns a cell array containing all variables occuring in input argument
% str (of type string)
var={};

for j=1:numel(model.bool_species)
  if(~isempty(strfind(str,model.bool_species{j})))
      var=[var model.bool_species{j}];
  end
end
  
end


function h = subsTab(values,func)
% substitues the occuring variables in a function (second input argument) with
% the values, given in the first input argument (called values)
      var = get_var(func);
      h=func;
      
      for j=1:numel(var)
          h=subs(h, var{j},values(j));
          
      end
end

end


function next = g(sgn_vec,current) 
%the update function g 
next=zeros(1,numel(sgn_vec));
        for n=1:numel(sgn_vec)
            
            if sgn_vec(n)==0
                next(n)=current(n);
                
            else if sgn_vec(n) > 0
                    next(n) = 1;
                else next(n) = 0;
                end
            end
         
        end
    end



function arr = get_LogicVar(row,var_cell)
% substitutes the values in a table row with a logical variable
% if the value is zero, it gets substituted by the negated variable
% the output is a cell array 
    arr = {}; 
    for j=1:(numel(var_cell))
        if row.(var_cell{j})==0
            x = ['!' char(var_cell{j})]; % '!' for negation
        else x = char(var_cell{j});
        end
        arr = [arr x];
    end
end



function T = get_tTable(n)
% returns a matrix of size (2^n x n)
% example call with n=2:
    T=[];
    d=(0:2^n-1).'; %colomn vector of size 2^n with entries: 0,..2^n-1
                   %for n=3, here d=(0 1 2 3)'
    for j=1:n
       
       t=(rem(floor(d*pow2(1-j)),2));
       % returns the remainder after division of floor(d*pow2(1-j)) by 2
       
       % for j=1,n=2:   t=rem((0 1 2 3),2)=(0 1 0 1)
       % for j=2,n=2:   t=rem(floor(0 0.5 1 1.5),2)=(0 0 1 1)
       T = [t T];   %inserts the colomn vector in front of matrix T
    end
    
end



